=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2007-04-11 22:05:16 +0000
+++ bzrlib/diff.py	2007-07-09 07:38:03 +0000
@@ -436,7 +436,7 @@
     has_changes = 0
     for path, file_id, kind in delta.removed:
         has_changes = 1
-        print >>to_file, '=== removed %s %r' % (kind, path.encode('utf8'))
+        print >>to_file, "=== removed %s '%s'" % (kind, path.encode('utf8'))
         old_name = '%s%s\t%s' % (old_label, path,
                                  _patch_header_date(old_tree, file_id, path))
         new_name = '%s%s\t%s' % (new_label, path, EPOCH_DATE)
@@ -444,7 +444,7 @@
                                          new_name, None, None, to_file)
     for path, file_id, kind in delta.added:
         has_changes = 1
-        print >>to_file, '=== added %s %r' % (kind, path.encode('utf8'))
+        print >>to_file, "=== added %s '%s'" % (kind, path.encode('utf8'))
         old_name = '%s%s\t%s' % (old_label, path, EPOCH_DATE)
         new_name = '%s%s\t%s' % (new_label, path,
                                  _patch_header_date(new_tree, file_id, path))
@@ -455,7 +455,7 @@
          text_modified, meta_modified) in delta.renamed:
         has_changes = 1
         prop_str = get_prop_change(meta_modified)
-        print >>to_file, '=== renamed %s %r => %r%s' % (
+        print >>to_file, "=== renamed %s '%s' => %r%s" % (
                     kind, old_path.encode('utf8'),
                     new_path.encode('utf8'), prop_str)
         old_name = '%s%s\t%s' % (old_label, old_path,
@@ -470,7 +470,8 @@
     for path, file_id, kind, text_modified, meta_modified in delta.modified:
         has_changes = 1
         prop_str = get_prop_change(meta_modified)
-        print >>to_file, '=== modified %s %r%s' % (kind, path.encode('utf8'), prop_str)
+        print >>to_file, "=== modified %s '%s'%s" % (kind, path.encode('utf8'),
+                                                     prop_str)
         # The file may be in a different location in the old tree (because
         # the containing dir was renamed, but the file itself was not)
         old_path = old_tree.id2path(file_id)

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2007-07-17 20:04:13 +0000
+++ bzrlib/inventory.py	2007-07-26 21:18:35 +0000
@@ -665,7 +665,8 @@
                 label_pair = (to_label, from_label)
             else:
                 label_pair = (from_label, to_label)
-            print >> output_to, "Binary files %s and %s differ" % label_pair
+            print >> output_to, \
+                  ("Binary files %s and %s differ" % label_pair).encode('utf8')
 
     def has_text(self):
         """See InventoryEntry.has_text."""

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2007-07-22 17:19:18 +0000
+++ bzrlib/tests/__init__.py	2007-07-26 21:18:35 +0000
@@ -1640,7 +1640,7 @@
         """Return a writeable transport.
 
         This transport is for the test scratch space relative to
-        "self._test_root""
+        "self._test_root"
         
         :param relpath: a path relative to the base url.
         """

=== modified file 'bzrlib/tests/test_diff.py'
--- bzrlib/tests/test_diff.py	2007-04-11 22:05:16 +0000
+++ bzrlib/tests/test_diff.py	2007-07-26 21:18:35 +0000
@@ -24,10 +24,39 @@
 from bzrlib.errors import BinaryFile, NoDiff
 import bzrlib.osutils as osutils
 import bzrlib.patiencediff
-from bzrlib.tests import (TestCase, TestCaseWithTransport,
+from bzrlib.tests import (Feature, TestCase, TestCaseWithTransport,
                           TestCaseInTempDir, TestSkipped)
 
 
+class _UnicodeFilename(Feature):
+    """Does the filesystem support Unicode filenames?"""
+
+    def _probe(self):
+        try:
+            os.stat(u'\u03b1')
+        except UnicodeEncodeError:
+            return False
+        except (IOError, OSError):
+            # The filesystem allows the Unicode filename but the file doesn't
+            # exist.
+            return True
+        else:
+            # The filesystem allows the Unicode filename and the file exists,
+            # for some reason.
+            return True
+
+UnicodeFilename = _UnicodeFilename()
+
+
+class TestUnicodeFilename(TestCase):
+
+    def test_probe_passes(self):
+        """UnicodeFilename._probe passes."""
+        # We can't test much more than that because the behaviour depends
+        # on the platform.
+        UnicodeFilename._probe()
+        
+
 def udiff_lines(old, new, allow_binary=False):
     output = StringIO()
     internal_diff('old', old, 'new', new, output, allow_binary)
@@ -441,6 +470,34 @@
         self.assertContainsRe(diff, '-contents\n'
                                     '\\+new contents\n')
 
+    def test_binary_unicode_filenames(self):
+        """Test that contents of files are *not* encoded in UTF-8 when there
+        is a binary file in the diff.
+        """
+        # See https://bugs.launchpad.net/bugs/110092.
+        self.requireFeature(UnicodeFilename)
+
+        # This bug isn't triggered with cStringIO.
+        from StringIO import StringIO
+        tree = self.make_branch_and_tree('tree')
+        alpha, omega = u'\u03b1', u'\u03c9'
+        alpha_utf8, omega_utf8 = alpha.encode('utf8'), omega.encode('utf8')
+        self.build_tree_contents(
+            [('tree/' + alpha, chr(0)),
+             ('tree/' + omega,
+              ('The %s and the %s\n' % (alpha_utf8, omega_utf8)))])
+        tree.add([alpha], ['file-id'])
+        tree.add([omega], ['file-id-2'])
+        diff_content = StringIO()
+        show_diff_trees(tree.basis_tree(), tree, diff_content)
+        diff = diff_content.getvalue()
+        self.assertContainsRe(diff, r"=== added file '%s'" % alpha_utf8)
+        self.assertContainsRe(
+            diff, "Binary files a/%s.*and b/%s.* differ\n" % (alpha_utf8, alpha_utf8))
+        self.assertContainsRe(diff, r"=== added file '%s'" % omega_utf8)
+        self.assertContainsRe(diff, r"--- a/%s" % (omega_utf8,))
+        self.assertContainsRe(diff, r"\+\+\+ b/%s" % (omega_utf8,))
+
 
 class TestPatienceDiffLib(TestCase):
 

